package main

import (
	"encoding/json"
	"fmt"
	"github.com/denverdino/aliyungo/common"
	"github.com/denverdino/aliyungo/ecs"
	"io/ioutil"
	"os"
	"strings"
)

var (
	cli    *ecs.Client = nil
	config *Config     = new(Config)
)

func main() {
	var err error
	filename := os.Args[1]
	resBytes, err := ioutil.ReadFile(filename)
	if err != nil {
		panic(err)
	}

	if err := json.Unmarshal(resBytes, config); err != nil {
		panic(err)
	}
	config.IpList = strings.Split(config.SourceCidrIp, ",")

	cli = ecs.NewClient(config.AccessKeyId, config.AccessKeySecret)

	switch config.Action {
	case "add":
		add()
	case "remove":
		remove()
	case "info":
		info()
	default:
		panic("invalid action")
	}

	fmt.Println("success")
}

func add() {
	param := &ecs.AuthorizeSecurityGroupArgs{
		SecurityGroupId: config.SecurityGroupId,
		RegionId:        common.Region(config.RegionId),
		IpProtocol:      ecs.IpProtocol(config.Protocol),
		PortRange:       config.PortRange,
		Policy:          ecs.PermissionPolicyDrop,
		Priority:        config.Priority,
		NicType:         ecs.NicTypeInternet,
	}
	for _, ip := range config.IpList {
		param.SourceCidrIp = strings.TrimSpace(ip)
		if err := cli.AuthorizeSecurityGroup(param); err != nil {
			panic(err)
		}
	}
}

func remove() {
	param := &ecs.RevokeSecurityGroupArgs{
		AuthorizeSecurityGroupArgs: ecs.AuthorizeSecurityGroupArgs{
			SecurityGroupId: config.SecurityGroupId,
			RegionId:        common.Region(config.RegionId),
			IpProtocol:      ecs.IpProtocol(config.Protocol),
			PortRange:       config.PortRange,
			Policy:          ecs.PermissionPolicyDrop,
			Priority:        config.Priority,
			NicType:         ecs.NicTypeInternet,
		},
	}
	for _, ip := range config.IpList {
		param.AuthorizeSecurityGroupArgs.SourceCidrIp = strings.TrimSpace(ip)
		if err := cli.RevokeSecurityGroup(param); err != nil {
			panic(err)
		}
	}
}

func info() {
	param := &ecs.DescribeSecurityGroupAttributeArgs{
		SecurityGroupId: config.SecurityGroupId,
		RegionId:        common.Region(config.RegionId),
		NicType:         ecs.NicTypeInternet,
	}
	if res, err := cli.DescribeSecurityGroupAttribute(param); err != nil {
		panic(err)
	} else {
		resBytes, err := json.MarshalIndent(res, "", "\t")
		if err != nil {
			panic(err)
		}
		fmt.Println(string(resBytes))
	}
}

type Config struct {
	AccessKeyId     string   `json:"accesskeyid"`
	AccessKeySecret string   `json:"accesskeysecret"`
	SecurityGroupId string   `json:"securitygroupid"`
	RegionId        string   `json:"regionid"`
	SourceCidrIp    string   `json:"sourcecidrip"`
	PortRange       string   `json:"portrange"`
	Protocol        string   `json:"protocol"`
	Priority        int      `json:"priority"`
	IpList          []string `json:"-"`
	Action          string   `json:"action"`
}
